--5/17/11
--Neuron model component
--sends data to the adcDacController.
--Model output is produced by the audio codec DAC, and can be viewed on the oscilloscope
--by connecting to the DE1 audio line-out

LIBRARY ieee;
USE ieee.std_logic_1164.all;
use ieee.std_logic_signed.all;

entity neuronModel is
	port(
		frameClock: in std_logic;
		resetn: in std_logic;
		vOut: out std_logic_vector(17 downto 0);
		uOut: out std_logic_vector(17 downto 0)
	);
end neuronModel;

architecture behavior of neuronModel is

	component signedMultiplier is
		port(
			multiplier: in std_logic_vector(17 downto 0);
			multiplicand: in std_logic_vector(17 downto 0);
			product: out std_logic_vector(17 downto 0)
		);
	end component;

	--state variables
	signal v, vNew, vSquared, vdt, u, uNew, uReset, udt, vb, vbMinusU: std_logic_vector(17 downto 0) := '0'&'0'&X"0000";
	--parameters and constants
	signal a, b, c, d, p, c14, l: std_logic_vector(17 downto 0) := '0'&'0'&X"0000";
	--prescalar of 4096
	signal count: integer range 0 to 8 := 0;
	
begin

	--this is the synchronous state variable update,
	--at 12 KHz (8 counts of the 96 KHz frame clock)
	process(frameClock, resetn)
	begin
		--initialize constants
		a <= '0'&'0'&X"051E"; --18'h0051E
		b <= '0'&'0'&X"3333"; --18'h03333 = 0.2
		c <= '1'&'1'&X"8000"; -- 18'h260000= -0.625
		d <= '0'&'0'&X"051E"; -- 18'h01478 = 0.08
		p <= '0'&'0'&X"4CCC"; -- 18'h16666; = 1.4
		c14 <= '0'&'1'&X"6666"; -- 18'h16666; = 1.4
		l <= '0'&'0'&X"2666"; -- 18'h02666 = 0.15
		
		if resetn = '0' then
			count <= 0;
			-- initial state
			v <= '1'&'1'&X"4CCD"; -- 0x34CCD = -0.7
			u <= '1'&'1'&X"CCCD"; -- 0x3CCCD = -0.2
		else
			if rising_edge(frameClock) then
				if count = 8 then
					count <= 0;
				else
					count <= count + 1;
				end if;
				
				if count = 0 then
					if v > p then
						v <= c;
						u <= uReset;
					else
						v <= vNew;
						u <= uNew;
					end if;
				end if;
			end if;
		end if;
	end process;
	
	--Euler's method
	--dt = 1/16
	--Thus, time scale is actualy (12e3/16)^-1 = 1.3 ms
	--v1(n+1) = v1(n) + (v1(n)^2) + 5/4*v1(n) + 1.40/4 - u1(n)/4 + 1/4/4
	squareV: signedMultiplier port map(v, v, vSquared);
	vdt <= vSquared + (v + ((v(17)&v(17)&v(17 downto 2)))) + (c14(17)&c14(17)&c14(17 downto 2)) - 
		(u(17)&u(17)&u(17 downto 2)) + (l(17)&l(17)&l(17 downto 2));
	vNew <= v + (vdt(17)&vdt(17)&vdt(17 downto 2));
		
	--u1(n+1) = u1 + dt*a*(b*v1(n) - u1(n));
	vTimesb: signedMultiplier port map(v, b, vb);
	vbMinusU <= vb - u;
	axVbMinusU: signedMultiplier port map(a, vbMinusU, udt);
	uNew <= u + (udt(17)&udt(17)&udt(17 downto 2));
	uReset <= u + d;
	
	--state outputs
	vOut <= v;
	uOut <= u;
		
end behavior;