--5/17/11
--ModelSim test bench for the neuron project

LIBRARY ieee;
USE ieee.std_logic_1164.all;

entity neuron_projectTB is
end neuron_projectTB;

architecture testbench of neuron_projectTB is

	--Modelsim test signals
	--I2C ports
	signal I2C_SCLK: std_logic;
	signal I2C_SDAT: std_logic;
	--audio codec ports
	signal AUD_ADCDAT: std_logic;
	signal AUD_ADCLRCK: std_logic;
	signal AUD_DACLRCK: std_logic;
	signal AUD_DACDAT: std_logic;
	signal AUD_XCK: std_logic;
	signal AUD_BCLK: std_logic;
	--select loopback test or neuron model output
	signal SW: std_logic_vector(9 downto 0);
	
	--I2C controller to drive the Wolfson codec
	component audioCodecController is
	port(
		clock50MHz,reset: in std_logic;
		I2C_SCLK_Internal: out std_logic;
		--must be inout to allow FPGA to read the ack bit
		I2C_SDAT_Internal: out std_logic;
		SDAT_Control: out std_logic;
		--for testing
		clock50KHz_Out: out std_logic
	);
	end component;
	
	--waits 40ms, then asserts high output
	component delayCounter is
		port(
			clock,reset: in std_logic;
			--active high reset
			resetAdc: out std_logic
		);
	end component;
	
	--generates digital audio interface clock signals
	--starts after delayCounter asserts (40ms)
	component AdcDacController is
		port(
			--reset signal starts '0', then goes to '1' after 40 ms => active-low
			resetn: in std_logic;
			--from 50MHz PLL at toplevel
			clock18MHz_in: in std_logic;
			--line-in on the DE1
			adcData: in std_logic;
			--line-out on the DE1
			dacData: out std_logic;
			bitClock: out std_logic;
			dacLRSelect: out std_logic;
			adcLRSelect: out std_logic;
			--neuron model signals
			neuronVin: in std_logic_vector(15 downto 0);
			neuronUin: in std_logic_vector(15 downto 0);
			--selects loopback or neuron signals using switches
			outputSelect: in std_logic_vector(1 downto 0)
		);
	end component;
	
	--generates the neuron model signals using euler's method
	component neuronModel is
		port(
			frameClock: in std_logic;
			resetn: in std_logic;
			vOut: out std_logic_vector(17 downto 0);
			uOut: out std_logic_vector(17 downto 0)
		);
	end component;
	
	--clock signal from the PLL clockBuffers
	signal clock50MHz: std_logic := '0';
	
	--18MHz PLL output signal
	signal clock18MHz: std_logic := '0';
	
	--asynchronous reset for the whole project
	signal reset: std_logic;
 
	--I2C data and clock lines
	signal i2cData, i2cClock: std_logic;
	
	--tri-state buffer control
	signal i2cDataControl: std_logic;
	signal i2cDataTriState: std_logic;
	
	--assert signal from delay counter
	signal codecResetn: std_logic;
	
	--audio codec signals
	signal adcDat_sig: std_logic;
	signal adcLRCK_sig: std_logic;
	signal dacLRCK_sig: std_logic;
	signal dacDat_sig: std_logic;
	signal bck_sig: std_logic;
	
	--neuron model signals
	signal vNeuronSig: std_logic_vector(17 downto 0);
	signal uNeuronSig: std_logic_vector(17 downto 0);
	signal outputSelect: std_logic_vector(1 downto 0);
	
	--for testing
	signal clock50KHz: std_logic;
	
begin

	--selects loopback test or neuron model output
	outputSelect <= SW(1 downto 0);
	
	--I2C
	I2CControllerInstance: audioCodecController port map(clock50MHz, reset, i2cClock, i2cData,
		i2cDataControl, clock50KHz);
		
	--Delay counter
	delayCounterMap: delayCounter port map(clock50MHz, reset, codecResetn);
	
	--Codec Controller
	AdcDacControllerMap: AdcDacController port map(codecResetn, clock18MHz, adcDat_sig, dacDat_sig,
		bck_sig, dacLRCK_sig, adcLRCK_sig, vNeuronSig(17 downto 2), uNeuronSig(17 downto 2), outputSelect);
		
	--neuron model
	neuronModelMap: neuronModel port map(adcLRCK_sig, codecResetn, vNeuronSig, uNeuronSig);
	
	--tri-state data output
	i2cDataTriState <= i2cData when i2cDataControl = '1' else 'Z';
	
	--I2C output ports
	I2C_SCLK <= i2cClock;
	I2C_SDAT <= i2cDataTriState;
	
	--audio codec input port
	adcDat_sig <= AUD_ADCDAT;
	
	--audio codec ouput ports
	AUD_ADCLRCK <= adcLRCK_sig;
	AUD_DACLRCK <= dacLRCK_sig;
	AUD_DACDAT <= dacDat_sig;
	AUD_XCK <= clock18MHz;
	AUD_BCLK <= bck_sig;

	--simulation stimulus
	process
	begin
		--wait 1/2 clock cycle (50MHz clock)
		wait for 10 ns;
		clock50MHz <= not clock50MHz;
	end process;
	
	process
	begin
		--wait 1/2 clock cycle (~18MHz clock)
		wait for 28 ns;
		clock18MHz <= not clock18MHz;
	end process;
	
	--temporary reset at start
	process
	begin
		--active high reset
		reset <= '1';
		--enable neuron output
		SW <= "0000000010";
		 --1 clock cycle
		wait for 20 ns;
		reset <= '0';
		--wait forever
		wait;
	end process;	
		
end testbench;